@@ -22,6 +22,7 @@ import re
 import typing
 from functools import lru_cache
 from urllib.parse import urlparse
+from datetime import datetime
 
 import attr
 import requests
@@ -36,14 +37,22 @@ from .exceptions import VersionIdentifierError
 from .configuration import config
 from .artifact import Artifact
 
-_LOGGER = logging.getLogger(__name__)
+from thoth.common.helpers import parse_datetime
 
+_LOGGER = logging.getLogger(__name__)
+LEGACY_URLS = {"https://pypi.python.org/simple": "https://pypi.org/simple"}
+
+def normalize_url(url:str) -> str:
+    """We normalize url to remove legacy urls."""
+    if url in LEGACY_URLS:
+       return LEGACY_URLS[url]
+    return url
 
 @attr.s(frozen=True, slots=True)
 class Source:
     """Representation of source (Python index) for Python packages."""
 
-    url = attr.ib(type=str)
+    url = attr.ib(type=str, converter=normalize_url)
     name = attr.ib(type=str)
     verify_ssl = attr.ib(type=bool, default=True)
     warehouse = attr.ib(type=bool)
@@ -241,10 +250,10 @@ class Source:
         package_info = self._warehouse_get_api_package_info(package_name)
         return list(package_info["releases"].keys())
 
-    def get_latest_package_version(
-        self, package_name: str, graceful: bool = False
-    ) -> typing.Optional[typing.Union[Version, LegacyVersion]]:
-        """Get the latest version for the given package."""
+    def get_sorted_package_versions(
+        self, package_name: str, graceful: bool = False, reverse: bool = True,  # default to newest first
+    ) -> typing.Optional[typing.List[typing.Union[Version, LegacyVersion]]]:
+        """Get sorted versions for the given package."""
         try:
             all_versions = self.get_package_versions(package_name)
         except NotFound:
@@ -266,8 +275,15 @@ class Source:
                 raise VersionIdentifierError(error_msg) from exc
 
             semver_versions.append(version)
+        return sorted(semver_versions, reverse=reverse)
 
-        return sorted(semver_versions)[-1]
+    def get_latest_package_version(
+        self, package_name: str, graceful: bool = False
+    ) -> typing.Optional[typing.Union[Version, LegacyVersion]]:
+        """Get the latest version for the given package."""
+
+        semver_versions = self.get_sorted_package_versions(package_name=package_name, graceful=graceful)
+        return semver_versions[0]
 
     def _simple_repository_list_artifacts(self, package_name: str) -> list:
         """Parse simple repository package listing (HTML) and return artifacts present there."""
@@ -405,3 +421,17 @@ class Source:
             result.append(doc)
 
         return result
+
+    def get_package_release_date(self, package_name: str, package_version: str,) -> datetime:
+        """Get time at which package was uploaded to package index."""
+        package_json = self._warehouse_get_api_package_info(package_name)
+        release = package_json["releases"].get(package_version)
+        if release is None:
+            raise NotFound(f"Version {package_version} not found for {package_name} on {self.warehouse_api_url}.")
+        artifact = next(x for x in release if x["python_version"] == "source")
+        if artifact is None:
+            raise NotFound(
+                f"No source distribution for {package_name}==={package_version} found on {self.warehouse_api_url}."
+            )
+
+        return parse_datetime(artifact["upload_time_iso_8601"][:-1])
